'use strict';

let AWS = require('aws-sdk');
let _ = require('lodash');
let async = require('async');
let CacheHelper = require('./redis_helper.js');
let DynamodbHelper = require('./dynamodb_helper.js');

/**
 * Search and retrieve configurations
 */
class ConfigurationManager {
    /**
     * @constructor
     * @param {object=} params additional configuration options
     */
    constructor(params) {
        if (typeof(params) == 'object' && params.hasOwnProperty('source')) {
            if (params.source.hasOwnProperty('name')) {
                this.source = params.source;
            } else {
                throw "No source name is specified";
            }
        } else {
            this.source = {name: `env_${process.env.NODE_ENV}`};
        }

        let creds = null;
        if (typeof(params) == 'object' && params.hasOwnProperty('profile')) {
            creds = new AWS.SharedIniFileCredentials({profile: params.profile});
        } else if (process.env.hasOwnProperty('AWS_PROFILE')) {
            creds = new AWS.SharedIniFileCredentials({profile: process.env.aws_profile});
        } else {
            creds = new AWS.EnvironmentCredentials('AWS');
        }

        this._awsConfig = {
            credentials: creds,
            region: 'ap-southeast-2'
        };

        this._ssm = new AWS.SSM(this._awsConfig);
        this._kms = new AWS.KMS(this._awsConfig);
        
        this.cache = {};
        this.cacheHelper = new CacheHelper();
    }

    /**
     * Get credentials from internal cache, redis, or dynamodb
     * 
     * @param {*} klass 
     * @param {*} instanceOf 
     * @param {*} cb 
     */
    getCredentials(klass, instanceOf, cb) {
        let cacheKey = this.makeCacheKey(klass, instanceOf);

        if(_.isEmpty(this.cache[cacheKey])) {
            //this.getCredentialsFromDb(klass, instanceOf, cb);
            this.getCredentialsFromDDB(klass, instanceOf, (err, info) => {
                if (err) {
                    cb(err, null);
                } else {
                    this.getParameters(info, cb);
                }
            });
        } else {
            this.getParameters(this.cache[cacheKey], cb);
        }
    }

    /**
     * Get credentials from redis or dynamodb
     * 
     * @param {*} klass 
     * @param {*} instanceOf 
     * @param {*} cb 
     */
    getCredentialsFromDb(klass, instanceOf, cb){
        this.cacheHelper.enter((err, res) => {
            if (err) {
                // goto DDB
                this.getCredentialsFromDDB(klass, instanceOf, cb);
            } else {
                // check cache
                let cacheKey = this.makeCacheKey(klass, instanceOf);
                this.cacheHelper.getValue(cacheKey, (err, res) => {
                    if (err) {
                        // goto DDB
                        this.cacheHelper.end((err, res) => {
                            this.getCredentialsFromDDB(klass, instanceOf, cb);
                        });
                    } else {
                        if (res === 'KEY MISS') {
                            this.getCredentialsFromDDB(klass, instanceOf, (err, info) => {
                                if(err) {
                                    this.cacheHelper.end((err, ret) => {
                                        return cb(err, null);
                                    });
                                } else {
                                    this.cacheHelper.setValue(cacheKey, JSON.stringify(info), 30, (err, res) => {
                                        this.cacheHelper.end((err, ret) => {
                                            return cb(null, info);
                                        });
                                    });
                                }
                            });
                        } else {
                            this.cacheHelper.end((err, ret) => {
                                return cb(null, JSON.parse(res));
                            });
                        }
                    }
                });
            }
        });
    }

    /**
     * Get credentials from dynamodb
     * 
     * @param {*} klass 
     * @param {*} instanceOf 
     * @param {*} cb 
     */
    getCredentialsFromDDB(klass, instanceOf, cb) {
        let dynamodb = new DynamodbHelper(this.source);
        let table = dynamodb.getTable(this.source.name);
        let key = {
            klass: klass,
            instanceof: instanceOf
        };

        table.getItem(key, null, (err, data) => {
            if (err) return cb(err, null);

            if (data.hasOwnProperty('Item')) {
                let record = data.Item;
                let cacheKey = this.makeCacheKey(klass, instanceOf);
                this.cache[cacheKey] = record.info;
                return cb(null, record.info);
            } else {
                return cb("No credential found", null);
            }
        });
    }

    /**
     * Make an unique key
     * 
     * @param {*} klass 
     * @param {*} instanceOf 
     */
    makeCacheKey(klass, instanceOf) {
        return `${klass}_${instanceOf}`;
    }

    getParameters(info, cb) {
        async.eachOf(
            info,
            (item, key, done) => {
                if (_.get(item, 'type', null) == 'parameter') {
                    let itemName = _.get(item, 'name', null);
                    let itemEncrypted = _.get(item, 'encrypted', false);
                    let ssmParam = {
                        "Name": itemName,
                        "WithDecryption": true
                    };
                    this._ssm.getParameter(ssmParam, (err, data) => {
                        if (err) {
                            done(err);
                        } else {
                            if (itemEncrypted) {
                                let kmsParam = {
                                    "CiphertextBlob" : Buffer.from(data.Parameter.Value, 'base64')
                                };
                                this._kms.decrypt(kmsParam, (err, data) => {
                                    if (err) {
                                        done(err);
                                    } else {
                                        info[key] = data.Plaintext.toString('utf-8');
                                        done(null);
                                    }
                                });
                            } else {
                                info[key] = data.Parameter.Value;
                                done(null);
                            }
                        }
                    });
                } else {
                    done(null);
                }
            },
            (err) => {
                if (err) {
                    cb(err, null);
                } else {
                    cb(null, info);
                }
            }
        );
    }
}

module.exports = ConfigurationManager;