'use strict';

let AWS = require('aws-sdk');
let DyanmodbTable = require('./dynamodb_table.js');

/**
 * Helper class for AWS DynamoDB
 */
class DynamodbHelper {
    /**
     * @constructor
     * @param {object=} params additional configuration options
     * @param {string=} profile aws profile name. If this is not specified, the profile name specified in AWS_PROFILE envrinment variable will be used. If AWS_PROFILE environment is not available, credentials will be retrieved from environment variables.
     * @param {string=} endpoint DynamoDB Endpoint URL. If this is specified, dynamodb cliet will try to connect to the specified endpoint url.
     * @param {string=} region The default value is 'ap-southeast-2'. If this is specified, it will use it instead.
     */
    constructor(params) {
        let creds = null;
        if (typeof(params) == 'object' && params.hasOwnProperty('profile')) {
            creds = new AWS.SharedIniFileCredentials({profile: params.profile});
        } else if (process.env.hasOwnProperty('AWS_PROFILE')) {
            creds = new AWS.SharedIniFileCredentials({profile: process.env.aws_profile});
        } else {
            creds = new AWS.EnvironmentCredentials('AWS');
        }

        this._awsConfig = {
            credentials: creds,
            region: 'ap-southeast-2'
        };

        this._dynamoDB = null;
        this._dynamoDBClient = null;
        this._tables = {};
    }

    /**
     * Create a AWS.DynamoDB object and then return it
     * 
     * @public
     * @returns {object}
     */
    getDynamoDB() {
        if (!this._dynamoDB) {
            this._dynamoDB = new AWS.DynamoDB(this._awsConfig);
        }
        return this._dynamoDB;
    }

    /**
     * Create a AWS.DynamoDB.DocumentClient object and then return it
     * 
     * @public
     * @returns {object}
     */
    getDynamoDBClient() {
        if (!this._dynamoDBClient) {
            this._dynamoDBClient = new AWS.DynamoDB.DocumentClient(this._awsConfig);
        }
        return this._dynamoDBClient;
    }

    /**
     * create a DynamodbTable object and return it
     * 
     * @public
     * @param {string} tableName name of the bucket that you want to work with
     * @returns {object} 
     */
    getTable(tableName) {
        let table = null;
        if (this._tables.hasOwnProperty(tableName)) {
            table = this._tables[tableName];
        } else {
            table = new DyanmodbTable(tableName, this);
        }
        return table;
    }
}

module.exports = DynamodbHelper;
