'use strict';

let AWS = require('aws-sdk');
let _ = require('lodash');

class DynamodbTable {

    /**
     * @constructor
     * @param {string} tableName 
     * @param {object} docClient AWS.DyanamoDB.DocumentClient Object
     */
    constructor(tableName, helper) {
        this._tableName = tableName;
        this._helper = helper;
        this._tableDescription = null;
    }

    /**
     * Retrieve a item
     * 
     * @public
     * @param {object} key 
     * @param {object} extraParams 
     * @param {function} cb 
     */
    getItem(key, extraParams, cb) {
        let params = {
            TableName: this._tableName,
            Key: key
        };

        if (extraParams) {
            params = _.assign(params, extraParams);
        }

        this._helper.getDynamoDBClient().get(params, cb);
    }

    /**
     * Creates a new item, or replaces an old item with a new item
     * 
     * @public
     * @param {object} item 
     * @param {object} extraParams 
     * @param {function} cb 
     */
    putItem(item, extraParams, cb) {
        let params = {
            TableName: this._tableName,
            Item: item
        };

        if (extraParams) {
            params = _.assign(params, extraParams);
        }

        this._helper.getDynamoDBClient().put(params, cb);
    }

    /**
     * Creates a new item, or ignore the request if a new item exist already
     * 
     * @public
     * @param {object} item 
     * @param {object} extraParams 
     * @param {function} cb 
     */
    postItem(item, extraParams, cb) {
        this.getTableDescription((err, data) => {
            if (err) {
                cb(err, null);
            } else {
                let partitionKey = this._getPrimaryPartitionKey();
                let sortKey = this._getPrimarySortKey();

                let conditionExpression = "#a <> :a";
                let expressionAttributeNames = {
                    '#a': partitionKey
                };
                let expressionAttributeValues = {
                    ':a': item[partitionKey]
                };

                if (sortKey) {
                    conditionExpression = conditionExpression + " and #b <> :b";
                    expressionAttributeNames['#b'] = sortKey;
                    expressionAttributeValues[':b'] = item[sortKey];
                }

                extraParams = _.assign(extraParams, {
                    ConditionExpression: conditionExpression, // this means do not overwrite this one if it already exists
                    ExpressionAttributeNames: expressionAttributeNames,
                    ExpressionAttributeValues: expressionAttributeValues
                });

                this.putItem(item, extraParams, cb);
            }
        });
    }

    /**
     * Delete a item
     * 
     * @public
     * @param {object} key 
     * @param {object} extraParams 
     * @param {function} cb 
     */
    deleteItem(key, extraParams, cb) {
        key = AWS.DynamoDB.Converter.marshall(key);

        let params = {
            TableName: this._tableName,
            Key: key
        };

        if (extraParams) {
            params = _.assign(params, extraParams);
        }

        this._helper.getDynamoDB().deleteItem(params, cb);
    }

    /**
     * Scan through all items
     * 
     * @param {string} select 
     * @param {string} lastEvaluatedKey 
     * @param {object} extraParams 
     * @param {function} cb 
     */
    scan(select, lastEvaluatedKey, filterExpression, filterValues, extraParams, cb) {
        let params = {
            TableName: this._tableName
        };

        if (select) {
            params['ProjectionExpression'] = select
        }

        if (filterExpression && filterValues) {
            params['FilterExpression'] = filterExpression;
            params['ExpressionAttributeValues'] = AWS.DynamoDB.Converter.marshall(filterValues);
        }

        if (lastEvaluatedKey) {
            params['ExclusiveStartKey'] = AWS.DynamoDB.Converter.marshall(lastEvaluatedKey);
        }

        if (extraParams) {
            params = _.assign(params, extraParams);
        }

        this._helper.getDynamoDB().scan(params, (err, data) => {
            if (err) {
                cb(err, null);
            } else {
                for (let i=0; i<data.Items.length; i++) {
                    data.Items[i] = AWS.DynamoDB.Converter.unmarshall(data.Items[i]);
                }

                if (data.hasOwnProperty('LastEvaluatedKey')) {
                    data['LastEvaluatedKey'] = AWS.DynamoDB.Converter.unmarshall(data['LastEvaluatedKey']);
                }

                cb(null, data);
            }
        });
    }

    query(select, lastEvaluatedKey, conditionExpression, conditionValues, extraParams, cb) {
        let params = {
            TableName: this._tableName
        };

        if (select) {
            params['ProjectionExpression'] = select
        }

        if (conditionExpression && conditionValues) {
            params['KeyConditionExpression'] = conditionExpression;
            params['ExpressionAttributeValues'] = AWS.DynamoDB.Converter.marshall(conditionValues);
        }

        if (lastEvaluatedKey) {
            params['ExclusiveStartKey'] = AWS.DynamoDB.Converter.marshall(lastEvaluatedKey);
        }

        if (extraParams) {
            params = _.assign(params, extraParams);
        }

        this._helper.getDynamoDB().query(params, (err, data) => {
            if (err) {
                cb(err, null);
            } else {
                //console.log(data);
                for (let i=0; i<data.Items.length; i++) {
                    data.Items[i] = AWS.DynamoDB.Converter.unmarshall(data.Items[i]);
                }

                if (data.hasOwnProperty('LastEvaluatedKey')) {
                    data['LastEvaluatedKey'] = AWS.DynamoDB.Converter.unmarshall(data['LastEvaluatedKey']);
                }

                cb(null, data);
            }
        });
    }

    /**
     * Retrive the description of the table
     * 
     * @public
     * @param {function} cb 
     */
    getTableDescription(cb) {
        if (this._tableDescription) {
            cb(null, this._tableDescription);
        } else {
            let params = {
                TableName: this._tableName
            };
            this._helper.getDynamoDB().describeTable(params, (err, data) => {
                if (err) {
                    cb(err, null)
                } else {
                    this._tableDescription = data;
                    cb(null, data);
                }
            });
        }
    }

    /**
     * Retrieve the primary partition key from the table description. getTableDescription() should be called prior to this function.
     * 
     * @private
     * @returns {string} primary partition key name
     */
    _getPrimaryPartitionKey() {
        if (this._tableDescription) {
            let key = null;
            if (this._tableDescription.hasOwnProperty('Table') && this._tableDescription.Table.hasOwnProperty('KeySchema')) {
                for(let i=0; i<this._tableDescription.Table.KeySchema.length; i++) {
                    if (this._tableDescription.Table.KeySchema[i].KeyType == 'HASH') {
                        key = this._tableDescription.Table.KeySchema[i];
                        break;
                    }
                }
            }

            return key ? key.AttributeName : null;
        } else {
            throw "No table description found";
        }
    }

    /**
     * Retrieve the primary sort key from the table description. getTableDescription() should be called prior to this function.
     * 
     * @private
     * @returns {string} primary sort key name
     */
    _getPrimarySortKey() {
        if (this._tableDescription) {
            let key = null;
            if (this._tableDescription.hasOwnProperty('Table') && this._tableDescription.Table.hasOwnProperty('KeySchema')) {
                for(let i=0; i<this._tableDescription.Table.KeySchema.length; i++) {
                    if (this._tableDescription.Table.KeySchema[i].KeyType == 'RANGE') {
                        key = this._tableDescription.Table.KeySchema[i];
                        break;
                    }
                }
            }

            return key ? key.AttributeName : null;
        } else {
            throw "No table description found";
        }
    }
}

module.exports = DynamodbTable;