'use strict';

const request = require('request');
const _ = require('lodash');
const moment = require('moment');

class JuniferHelper {
    /**
     * @constructor
     * @param {object} _configurationManager
     */
    constructor(config) {
        this.config = config;

        this.config['apiVersion'] = _.has(this.config, 'apiVersion') ? this.config['apiVersion']: 'v1';
        this.config['timeout'] = _.has(this.config, 'timeout') ? this.config['timeout']: 15000;
        this.config['gcis'] = _.has(this.config, 'gcis') ? this.config['gcis']: false;
        this.token = null;
    }

    /**
     * Depends on GCIS setup, authentication method and endpoint url are different.
     * For Junifer MOJO instance, a request uses user id and passwod.
     * For Junifer PE instance, a request uses token so you need to get a token first.
     * 
     * @private
     * @param {string} path 
     * @param {object} getParams 
     * @param {object} postBody 
     * @param {string} methodOverride 
     * @param {callback} cb 
     */
    makeCall(path, params, postBody, methodOverride, cb) {
        if (this.config['gcis'] === true) {
            if (this.token && this.token['expiresAt'].isAfter(moment())) {
                this.makeCallThroughGCIS(path, params, postBody, methodOverride, cb);
            } else {
                this.getToken((err) => {
                    if (err) {
                        cb(err, null);
                    } else {
                        //console.log(this.token);
                        this.makeCallThroughGCIS(path, params, postBody, methodOverride, cb);
                    }
                });
            }
        } else {
            this.makeCallDirectly(path, params, postBody, methodOverride, cb);
        }
    }

    /**
     * Send a http request to Junifer
     * 
     * @private
     * @param {string} path 
     * @param {object} getParams 
     * @param {object} postBody 
     * @param {string} methodOverride 
     * @param {callback} cb 
     */
    makeCallDirectly(path, params, postBody, methodOverride, cb) {
        // setup common options
        var options = {
            url: `https://${this.config['hostname']}:${this.config['port']}/rest/${this.config['apiVersion']}/${path}`, 
            json: true,
            timeout: this.config['timeout'],
            time: true
        };
    
        // setup method and any body / qs params
        if (postBody) {
            options['body'] = postBody;
            options['method'] = 'POST';
        } else {
            options['method'] = 'GET';
            if (params) {
                options['qs'] = params;
            }
        }

        if (methodOverride) {
            options['method'] = methodOverride;
        }
    
        // add authentication tokens
        options['headers'] = {
            'X-Junifer-X-username': this.config['user'],
            'X-Junifer-X-apikey': this.config['apikey']
        };
        //console.log(options);
        //Mojo.UtilHelper.inspect(options, 'options');
    
        request(options, (error, response, body) => {
            if (error) {
                cb(this.buildErrorResponse('RequestNotSuccessful', error), null);
            } else {
                if (response.statusCode >= 200 && response.statusCode < 300) {
                    cb(null, body);
                } else {
                    cb(body, null);
                }
            }
        });
    }

    /**
     * Send a http request to Junifer
     * 
     * @param {string} path 
     * @param {object} params 
     * @param {object} postBody 
     * @param {string} methodOverride 
     * @param {callback} cb 
     */
    makeCallThroughGCIS(path, params, postBody, methodOverride, cb) {
        // setup common options
        let options = {
            url: `https://${this.config['hostname']}:${this.config['port']}/${this.config['apiVersion']}/junifer/${path}`, 
            json: true,
            timeout: this.config['timeout'],
            time: true
        };

        // setup method and any body / qs params
        if (postBody) {
            options['body'] = postBody;
            options['method'] = 'POST';
        } else {
            options['method'] = 'GET';
            if (params) {
                options['qs'] = params;
            }
        }

        if (methodOverride) {
            options['method'] = methodOverride;
        }
        //console.log(options);

        request(options, (error, response, body) => {
            if (error) {
                cb(this.buildErrorResponse('RequestNotSuccessful', error), null);
            } else {
                if (response.statusCode >= 200 && response.statusCode < 300) {
                    cb(null, body);
                } else {
                    cb(body, null);
                }
            }
        }).auth(null, null, true, this.token['accessToken']);
    }

    /**
     * Obtain a token
     * 
     * @param {callback} cb 
     */
    getToken(cb) {
        request.post(`https://${this.config['hostname']}:${this.config['port']}/${this.config['apiVersion']}/token`, (error, response, body) => {
            //console.log(body);
            if (error) {
                cb(this.buildErrorResponse('AuthenticationFailed', error));
            } else {
                if (response.statusCode >= 200 && response.statusCode < 300) {
                    this.token = JSON.parse(body);
                    this.token['expiresAt'] = moment().add(this.token['expiresIn'], 'seconds');
                    cb(null);
                } else {
                    cb(this.buildErrorResponse('AuthenticationFailed', body));
                }
            }
        }).auth(this.config['user'], this.config['apikey']);
    }

    /**
     * Return an error object that would be commpatible with Junifer's error object 
     * 
     * @private
     * @param {string} code 
     * @param {string} description 
     * @param {string} [severity='Error']
     * @returns {object}
     */
    buildErrorResponse(code, description, severity) {
        return {
            errorCode: code,
            errorSeverity: severity ? severity : 'Error',
            errorDescription: description
        };
    }

    /**
     * Get a customer searched by a customer id
     * 
     * @public
     * @param {number} customerId 
     * @param {callback} cb 
     */
    getCustomer(customerId, cb) {
        let path = `customers/${customerId}`;

        this.makeCall(path, null, null, null, (err, body) => {
            if (err) {
                cb(err, null);
            } else {
                if (_.has(body, 'id') && body.id == customerId) {
                    cb(null, body);
                } else {
                    cb(this.buildErrorResponse('InvalidResponseBody', JSON.stringify(body)));
                }
            }
        });
    }

    /**
     * Get an account searched by an account id
     * 
     * @public
     * @param {number} accountId 
     * @param {callback} cb 
     */
    getAccount(accountId, cb) {
        let path = `accounts/${accountId}`;
        this.makeCall(path, null, null, null, (err, body) => {
            if (err) {
                cb(err, null);
            } else {
                if (_.has(body, 'id') && body.id == accountId) {
                    cb(null, body);
                } else {
                    cb(this.buildErrorResponse('InvalidResponseBody', JSON.stringify(body)));
                }
            }
        });
    }

    /**
     * Get a customer searched by an customer number
     * 
     * @public
     * @param {number} customerNumber 
     * @param {callback} cb 
     */
    getCustomerByCustomerNumber(customerNumber, cb) {
        let path = `customers?number=${customerNumber}`;

        this.makeCall(path, null, null, null, (err, body) => {
            if (err) {
                cb(err, null);
            } else {
                if (_.has(body, 'results') && _.isArray(body.results) && body.results.length <= 1) {
                    if (body.results.length == 1) {
                        cb(null, body.results[0]);
                    } else {
                        cb(null, null);
                    }
                } else {
                    cb(this.buildErrorResponse('InvalidResponseBody', JSON.stringify(body)));
                }
            }
        });
    }
    
    /**
     * Get an account searched by an account number
     * 
     * @public
     * @param {number} accountNumber 
     * @param {callback} cb 
     */
    getAccountByAccountNumber(accountNumber, cb) {
        let path = `accounts/accountNumber/${accountNumber}`;

        this.makeCall(path, null, null, null, (err, body) => {
            if (err) {
                cb(err, null);
            } else {
                if (_.has(body, 'number') && body.number == accountNumber) {
                    cb(null, body);
                } else {
                    cb(this.buildErrorResponse('InvalidResponseBody', JSON.stringify(body)));
                }
            }
        });
    }

    /**
     * Search accounts by given parameters
     * 
     * @public
     * @param {object} params 
     * @param {callback} cb 
     */
    searchAccounts(params, cb) {
        let path = 'accounts';

        this.makeCall(path, params, null, null, (err, body) => {
            if (err) {
                cb(err, null);
            } else {
                if (_.has(body, 'results') && _.isArray(body.results)) {
                    cb(null, body);
                } else {
                    cb(this.buildErrorResponse('InvalidResponseBody', JSON.stringify(body)));
                }
            }
        });
    }

    /**
     * Get agreements for the given account
     * 
     * @public
     * @param {number} accountId 
     * @param {callback} cb 
     */
    getAgreementsByAccountId(accountId, cb) {
        let path = `accounts/${accountId}/agreements`;

        this.makeCall(path, null, null, null, (err, body) => {
            if (err) {
                cb(err, null);
            } else {
                if (_.has(body, 'results') && _.isArray(body.results)) {
                    cb(null, body.results);
                } else {
                    cb(this.buildErrorResponse('InvalidResponseBody', JSON.stringify(body)));
                }
            }
        });
    }

    /**
     * Enrols customer for electricity and/or gas product. This is an Australia specific API
     * 
     * @param {object} postData 
     * @param {callback} cb 
     */
    enrolCustomer(postData, cb) {
        let path = 'au/customers/enrolCustomer';

        this.makeCall(path, null, postData, null, cb);
    }

    /**
     * Enrols customer for electricity and/or gas product. This is an Australia specific API
     * 
     * @param {object} postData 
     * @param {callback} cb 
     */
     enrolBusinessCustomer(postData, cb) {
        let path = 'au/customers/enrolBusinessCustomer';

        this.makeCall(path, null, postData, null, cb);
    }

    /**
     * Renew account
     * 
     * @param {number} accountId 
     * @param {object} postData 
     * @param {callback} cb 
     */
    renewAccount(accountId, postData, cb) {
        let path = `accounts/${accountId}/renewal`;
        this.makeCall(path, null, postData, null, cb);
    }

    /**
     * Renew account
     * 
     * @param {number} accountNumber 
     * @param {object} postData 
     * @param {callback} cb 
     */
    renewAccountByAccountNumber(accountNumber, postData, cb) {
        let path = `accounts/accountNumber/${accountNumber}/renewal`;
        this.makeCall(path, null, postData, null, cb);
    }

    /**
     * Get payment methods registered for the account specified by the ID
     * 
     * @param {number} accountId 
     * @param {callback} cb 
     */
    getPaymentMethod(accountId, cb) {
        let path = `accounts/${accountId}/paymentMethods`;
        
        //makeCall(path, params, postBody, methodOverride, cb) {
        this.makeCall(path, null, null, null, (err, body) => {
            if (err) {
                cb(err, null);
            } else {
                if (_.has(body, 'results') && _.isArray(body.results)) {
                    cb(null, body.results);
                } else {
                    cb(this.buildErrorResponse('InvalidResponseBody', JSON.stringify(body)));
                }
            }
        });
    }

    /**
     * Cancel, delete future account payment methods.
     * 
     * @param {number} paymentMethodId 
     * @param {string} paymentMethodType 
     * @param {callback} cb 
     */
    cancelPaymentMethod(paymentMethodId, paymentMethodType, cb) {
        let path = ``;

        if (paymentMethodType == 'Bpoint Direct Debit') {
            path = 'bpointDirectDebits';
        } else if (paymentMethodType == 'PayPal Direct Debit') {
            path = 'payPalDirectDebits';
        }

        path = path + '/' + paymentMethodId;

        this.makeCall(path, null, null, 'DELETE', cb);
    }

    /**
     * Creates a new payment method for the account specified in the account.id field.
     * 
     * @param {number} accountId 
     * @param {string} type 
     * @param {string} token 
     * @param {callback} cb 
     */
    createPaymentMethod(accountId, type, token, cb) {
        let path;
        let postData = {
            'account.id': accountId,
            'mandateReference': token
        };
        if (type == 'Credit Card') {
            path = 'au/bpointDirectDebits';
            postData['paymentType'] = 'CreditCard';
        } else if (type == 'Direct Debit') {
            path = 'au/bpointDirectDebits';
            postData['paymentType'] = 'BankAccount';
        } else if (type == 'PayPal') {
            path = 'au/payPalDirectDebits';
        } else {
            return cb('Not supported payment type');
        }

        this.makeCall(path, null, postData, null, cb);
    }

    /**
     * Creates a concession. This is an Australia specific API.
     * 
     * @param {object} postData 
     * @param {callback} cb 
     */
    createConcession(postData, cb) {
        let path = 'au/concessions/createConcession';
        
        this.makeCall(path, null, postData, null, cb);
    }

    /**
     * Add a discount to an account
     * @param {number} accountId 
     * @param {string} discountDfnName 
     * @param {date} fromDt 
     * @param {callback} cb 
     */
    addDiscount(accountId, discountDfnName, fromDt, cb) {
        let path = `au/accounts/${accountId}/discounts`;
        let postData = {
            "discountDfnName": discountDfnName,
            "fromDt": fromDt
        };
        this.makeCall(path, null, postData, null, cb);
    }

    /**
     * Put the specified account id under review with some flags such as bill suppression
     * 
     * @param {number} accountId 
     * @param {object} postData 
     * @param {callback} cb 
     */
    createAccountReviewPeriod(accountId, postData, cb) {
        let path = `accounts/${accountId}/accountReviewPeriods`;
        
        this.makeCall(path, null, postData, null, cb);
    }

    /**
     * Put the specified account number under review with some flags such as bill suppression
     * 
     * @param {number} accountId 
     * @param {object} postData 
     * @param {callback} cb 
     */
     createAccountReviewPeriodByAccountNumber(accountNumber, postData, cb) {
        let path = `accounts/accountNumber/${accountNumber}/accountReviewPeriods`;
        
        this.makeCall(path, null, postData, null, cb);
    }

    /**
     * Get account review period details of the specified account id
     * 
     * @param {number} accountId 
     * @param {callback} cb 
     */
    getAccountReviewPeriod(accountId, cb) {
        let path = `accounts/${accountId}/accountReviewPeriods`;
        
        this.makeCall(path, null, null, null, cb);
    }

    /**
     * Get account review period details of the specified account number
     * 
     * @param {number} accountNumber 
     * @param {callback} cb 
     */
     getAccountReviewPeriodByAccountNumber(accountNumber, cb) {
        let path = `accounts/accountNumber/${accountNumber}/accountReviewPeriods`;
        
        this.makeCall(path, null, null, null, cb);
    }

    /**
     * Cancel the account review period specified by account id and account review period id
     * 
     * @param {number} accountId 
     * @param {number} accountReviewPeriodId 
     * @param {callback} cb 
     */
    cancelAccountReviewPeriod(accountId, accountReviewPeriodId, cb) {
        let path = `accounts/${accountId}/cancelAccountReviewPeriod`;
        let params = {
            accountReviewPeriodId: accountReviewPeriodId
        };
        
        this.makeCall(path, params, null, 'delete', cb);
    }

    /**
     * Cancel the account review period specified by account number and account review period id
     * 
     * @param {number} accountId 
     * @param {number} accountReviewPeriodId 
     * @param {callback} cb 
     */
     cancelAccountReviewPeriodByAccountNumber(accountNumber, accountReviewPeriodId, cb) {
        let path = `accounts/accountNumber/${accountNumber}/cancelAccountReviewPeriod`;
        let params = {
            accountReviewPeriodId: accountReviewPeriodId
        };
        
        this.makeCall(path, params, null, 'delete', cb);
    }

    /**
     * Create account credit for the given account id
     * 
     * @param {number} accountId 
     * @param {number} grossAmount 
     * @param {string} salesTaxName 
     * @param {string} accountCreditReasonName 
     * @param {string} reference 
     * @param {string} description 
     * @param {callback} cb 
     */
    createAccountCredits(accountId, grossAmount, salesTaxName, accountCreditReasonName, reference, description, cb) {
        let path = `accounts/${accountId}/accountCredits`;
        let postData = {
            "grossAmount": grossAmount,
            "salesTaxName": salesTaxName,
            "accountCreditReasonName": accountCreditReasonName,
            "reference": reference,
            "description": description
        };
        this.makeCall(path, null, postData, null, cb);
    }

    /**
     * get account's credits
     * 
     * @param {number} accountId 
     * @param {callback} cb 
     */
    getAccountCredits(accountId, cb) {
        let path = `accounts/${accountId}/accountCredits`;
        
        this.makeCall(path, null, null, null, (err, body) => {
            if (err) {
                cb(err, null);
            } else {
                if (_.has(body, 'results') && _.isArray(body.results)) {
                    cb(null, body.results);
                } else {
                    cb(this.buildErrorResponse('InvalidResponseBody', JSON.stringify(body)));
                }
            }
        });
    }

    /**
     * Create account debit for the given account id
     * 
     * @param {number} accountId 
     * @param {number} grossAmount 
     * @param {string} salesTaxName 
     * @param {string} accountDebitReasonName 
     * @param {string} reference 
     * @param {string} description 
     * @param {callback} cb 
     */
    createAccountDebits(accountId, grossAmount, salesTaxName, accountDebitReasonName, reference, description, cb) {
        let path = `accounts/${accountId}/accountDebits`;
        let postData = {
            "grossAmount": grossAmount,
            "salesTaxName": salesTaxName,
            "accountDebitReasonName": accountDebitReasonName,
            "reference": reference,
            "description": description
        };
        this.makeCall(path, null, postData, null, cb);
    }

    /**
     * get account's debits
     * 
     * @param {number} accountId 
     * @param {callback} cb 
     */
    getAccountDebits(accountId, cb) {
        let path = `accounts/${accountId}/accountDebits`;
        
        this.makeCall(path, null, null, null, (err, body) => {
            if (err) {
                cb(err, null);
            } else {
                if (_.has(body, 'results') && _.isArray(body.results)) {
                    cb(null, body.results);
                } else {
                    cb(this.buildErrorResponse('InvalidResponseBody', JSON.stringify(body)));
                }
            }
        });
    }

    /**
     * Get account's properties. If no properties can be found an empty results array is returned.
     * 
     * @param {number} accountId 
     * @param {callback} cb 
     */
    getProperties(accountId, cb) {
        let path = `accounts/${accountId}/propertys`;
        
        this.makeCall(path, null, null, null, (err, body) => {
            if (err) {
                cb(err, null);
            } else {
                if (_.has(body, 'results') && _.isArray(body.results)) {
                    cb(null, body.results);
                } else {
                    cb(this.buildErrorResponse('InvalidResponseBody', JSON.stringify(body)));
                }
            }
        });
    }

    /**
     * Change the customer llinked to the account to the customer
     * 
     * @param {number} accountId 
     * @param {number} customerId 
     * @param {callback} cb 
     */
    changeCustomerLink(accountId, customerId, cb) {
        let path = `accounts/${accountId}/changeAccountCustomer`;
        let postData = {
            "id": customerId,
        };
        this.makeCall(path, null, postData, 'PUT', cb);
    }
}

module.exports = JuniferHelper;