'use strict';

let AWS = require('aws-sdk');
let _ = require('lodash');

class LambdaHelper {
    /**
     * @constructor
     * @param {object=} params additional configuration options
     * @param {string=} params.profile aws profile name. If this is not specified, the profile name specified in AWS_PROFILE envrinment variable will be used. If AWS_PROFILE environment is not available, credentials will be retrieved from environment variables.
     * @param {string=} params.region The default value is 'ap-southeast-2'. If this is specified, it will use it instead.
     */
    constructor(params) {
        let creds = null;
        if (typeof(params) == 'object' && params.hasOwnProperty('profile')) {
            creds = new AWS.SharedIniFileCredentials({profile: params.profile});
        } else if (process.env.hasOwnProperty('AWS_PROFILE')) {
            creds = new AWS.SharedIniFileCredentials({profile: process.env.aws_profile});
        } else {
            creds = new AWS.EnvironmentCredentials('AWS');
        }

        this._awsConfig = {
            credentials: creds,
            region: 'ap-southeast-2'
        };

        this._lambdaClient = new AWS.Lambda(this._awsConfig);
    }

    /**
     * invoke the specified lambda function
     * 
     * @param {string} service 
     * @param {object} payload 
     * @param {object} extraParams 
     * @param {callback} cb 
     */
    invoke(service, payload, extraParams, cb) {
        let params = {
            FunctionName: service,
            Payload: JSON.stringify(payload)
        };

        if (extraParams) {
            params = _.assign(params, extraParams);
        }

        this._lambdaClient.invoke(params, cb);
    }
}

module.exports = LambdaHelper;
