'use strict';

const util = require('util');
const SnsHelper = require('./sns_helper.js');

class LogHelper {
    constructor(options = {}) {
        this.topic = options.topic || 'application-error';
        this.appName = options.appName || process.env.npm_package_name || process.env.AWS_LAMBDA_FUNCTION_NAME || null;
        this.appVersion = options.appVersion || process.env.npm_package_version || process.env.AWS_LAMBDA_FUNCTION_VERSION || null;
        this.handlerName = options.handlerName || process.env._HANDLER || null;
        this.handlerEvent = options.handlerEvent || null;
        this.handlerContext = options.handlerContext || null;
        this.envName = options.envName || process.env.NODE_ENV;
        this.sns = null;
        
    }

    /**
     * This function will publish an error message to the topic.
     * 
     * @param {*} error error message or error object
     * @param {*} cb callback function
     */
    notifyError(error, cb) {
        let message = this._getErrorMessage(error);

        if (this.sns == null) {
            this.sns = new SnsHelper;
        }
        this.sns.publish(this.topic, message, cb);
    }

    /**
     * This function will printout an error message to stdout.
     * 
     * @param {*} error error message or error object
     */
    printFatalError(error) {
        let message = this._getErrorMessage(error);

        console.log('FATAL ERROR: %s', util.inspect(message, false, null, false));
    }

    /**
     * This function will return a structured error message object.
     * 
     * @param {*} error error message or error object.
     * @returns a structured error message object.
     */
    _getErrorMessage(error) {
        let message = {
            error: error.stack || error,
            env: this.envName,
            timestamp: new Date().toLocaleString()
        };
        if (this.appName !== null) {
            message['appName'] = this.appName;
        }
        if (this.appVersion !== null) {
            message['appVersion'] = this.appVersion;
        }
        if (this.handlerName !== null) {
            message['handlerName'] = this.handlerName;
        }
        if (this.handlerEvent !== null) {
            message['handlerEvent'] = this.handlerEvent;
        }
        if (this.handlerContext !== null) {
            message['handlerContext'] = this.handlerContext;
        }
        return message;
    }
}

module.exports = LogHelper;