'use strict';

let _ = require('lodash');
let fs = require('fs');

class S3Bucket {
    
    /**
     * @constructor
     * @param {string} bucketName bucket name
     * @param {object} helper S3Helper object
     */
    constructor(bucketName, helper) {
        this._bucketName = bucketName;
        this._helper = helper;
    }

    /**
     * List objects of the bucket
     * 
     * @param {*} prefix Limits the response to keys that begin with the specified prefix.
     * @param {*} marker last key of the previous listing. if this is set, the function will try to list from the given key in alphabetical order
     * @param {*} extraParams other parameters of AWS.s3.listObjects()
     * @param {*} cb function(err, data, marker)
     */
    listObjects(prefix, marker, extraParams, cb) {
        let params = {
            Bucket: this._bucketName
        };
        if (prefix) {
            params['Prefix'] = prefix;
        }
        if (marker) {
            params['Marker'] = marker;
        }
        if (extraParams) {
            params = _.assign(params, extraParams);
        }
        this._helper.getS3().listObjects(params, (err, data) => {
            if (err) {
                cb('There was an error (' + err.message + ') listing objects of ' + this._bucketName, null, null);
            } else {
                if (data.IsTruncated) {
                    cb(null, data.Contents, data.Contents[data.Contents.length-1].Key);
                } else {
                    cb(null, data.Contents, null);
                }
            }
        });
    }

    /**
     * Retrieve a file(object) from s3 by filename(key) and then call call back function with body string of the retrieved file object.
     * 
     * @public
     * @param {string} remoteFileKey file object name
     * @param {object} extraParams other parameters other than bucket and key
     * @param {function} cb callback function
     */
    getObject(remoteFileKey, extraParams, cb) {
        let params = {
            Bucket: this._bucketName,
            Key: remoteFileKey
        };

        if (extraParams) {
            params = _.assign(params, extraParams);
        }

        this._helper.getS3().getObject(params, cb);
    }

    putObject(remoteFileKey, extraParams, bodyString, cb) {
        let params = {
            Body: bodyString,
            Bucket: this._bucketName,
            Key: remoteFileKey
        };

        if (extraParams) {
            params = _.assign(params, extraParams);
        }

        this._helper.getS3().putObject(params, cb);
    }

    /**
     * Retrieve a file(object) from s3 by filename(key) and then streams it to a local file, with race conditions checked and managed.
     * 
     * @public
     * @param {string} remoteFileKey file object name
     * @param {object} localFilePath local temporary file name
     * @param {function} cb callback function
     */
    downloadFileLocal(remoteFileKey, localFilePath, cb) {
        var params = {
            Bucket: this._bucketName, 
            Key: remoteFileKey
        };

        var output = fs.createWriteStream(localFilePath);
        var input = this._helper.getS3().getObject(params).createReadStream();
        var closedRead = false;
        var closedWrite = false;
        var callbackCalled = false;

        input.on('end', () => {
            closedRead = true;
            if(closedWrite && !callbackCalled) {
                //console.log('output closed, then input closed');
                callbackCalled = true;
                cb(null, 'ok');
            }
        });

        output.on('finish', () => {
            closedWrite = true;
            if(closedRead && !callbackCalled) {
                //console.log('input closed, then output closed');
                callbackCalled = true;
                cb(null, 'ok');
            }
        });

        input.pipe(output);
    }

    /**
     * Retrieve a head of object; returns metadata of object as well as used to check if a) exists and b) have permissions to that object.
     * 
     * @public
     * @param {string} remoteFileKey file object name
     * @param {function} cb callback function
     */
    getObjectHead(remoteFileKey, cb) {
        var params = {
            Bucket: this._bucketName,
            Key: remoteFileKey
        };
        
        s3.headObject(params, function(err, data) {
            if (err){
                //console.log(JSON.stringify(err));
                cb(null, null);
            } else {
                cb(null, data);
            }     
        });
    }

    /**
     * Generate a signed url
     * 
     * @public
     * @param {string} remoteFileKey file object name
     * @param {*} ttl the signed url will be expired with in the specified ttl(in seconds).
     */
    getSignedUrl(remoteFileKey, ttl) {
        let params = {
            Bucket: this._bucketName, 
            Key: remoteFileKey, 
            Expires: ttl ? ttl : 60 * 60
        };
        let _signedUrl = this._helper.getS3().getSignedUrl('getObject', params);

        return _signedUrl;
    }

    /**
     * Uploads an arbitrarily sized buffer, blob, or stream, using intelligent concurrent handling of parts if the payload is large enough.
     * 
     * @public
     * @param {string} remoteFileKey 
     * @param {stream} stream 
     * @param {callback} cb 
     */
     upload(remoteFileKey, stream, cb) {
        let params = {
            Bucket: this._bucketName, 
            Key: remoteFileKey, 
            Body: stream
        };
        
        this._helper.getS3().upload(params, cb);
    }
}

module.exports = S3Bucket;
