'use strict';

let AWS = require('aws-sdk');

class SNSHelper {
    /**
     * @constructor
     */
    constructor(params) {
        let creds = null;
        if (typeof(params) == 'object' && params.hasOwnProperty('profile')) {
            creds = new AWS.SharedIniFileCredentials({profile: params.profile});
        } else if (process.env.hasOwnProperty('AWS_PROFILE')) {
            creds = new AWS.SharedIniFileCredentials({profile: process.env.aws_profile});
        } else {
            creds = new AWS.EnvironmentCredentials('AWS');
        }

        this._awsConfig = {
            credentials: creds,
            region: 'ap-southeast-2'
        };

        this._sns = null;
        this._topics = null;
    }

    /**
     * create AWS.S3 object
     * 
     * @public
     * @returns {object} AWS.S3 object
     */
    getSNS() {
        if (!this._sns) {
            this._sns = new AWS.SNS(this._awsConfig);
        }

        return this._sns;
    }

    listTopics(cb, nextToken) {
        let params = {};
        if (nextToken) {
            params['NextToken'] = nextToken;
        }
        this.getSNS().listTopics(params, (err, data) => {
            if (err) {
                cb(err, null);
            } else {
                if (this._topics) {
                    this._topics = this._topics.concat(data.Topics);
                } else {
                    this._topics = data.Topics;
                }
                if (data.hasOwnProperty('NextToken')) {
                    this.listTopics(cb, data.NextToken);
                } else {
                    cb(null, this._topics);
                }
            }
        });
    }

    getTopicArn(topic, cb) {
        let arn = null;
        if (!this.topics) {
            this.listTopics((err, data) => {
                if (err) {
                    cb(err, null);
                } else {
                    arn = this._getTopicArnFromTopicsProperty(topic);
                    
                    if (arn) {
                        cb(null, arn);
                    } else {
                        cb('cannot find the arn for the specified topic "'+topic+'"', null);
                    }
                }
            });
        } else {
            arn = this._getTopicArnFromTopicsProperty(topic);
                    
            if (arn) {
                cb(null, arn);
            } else {
                cb('cannot find the arn for the specified topic "'+topic+'"', null);
            }
        }
    }

    _getTopicArnFromTopicsProperty(topic) {
        let arn = null;
        for (let i=0; i<this._topics.length; i++) {
            if (this._topics[i].hasOwnProperty('TopicArn')) {
                let topicArnTokens = this._topics[i]['TopicArn'].split(":");
                if (topicArnTokens[5] == topic) {
                    arn = this._topics[i]['TopicArn'];
                    break;
                }
            }
            
        }
        return arn;
    }
    

    publish(topic, obj, cb) {
        let payload = JSON.stringify(obj);
        this.getTopicArn(topic, (err, arn) => {
            if (err) {
                cb(err, null);
            } else {
                this.getSNS().publish({
                    Message: payload,
                    //MessageStructure: 'json',
                    TargetArn: arn
                }, cb);
            }
        });
    }
}

module.exports = SNSHelper;
