var _ = require('lodash');
var mysql = require('mysql');
var SqlString = require('sqlstring');
var async = require('async');

let _helper = (function() {
    
    let configurationManager;

    // contains cluster of pools
    let poolCluster;

    // constructor
    let _helper = function(_configurationManager) {
        configurationManager = _configurationManager;
        poolCluster = mysql.createPoolCluster();
    };

    _helper.prototype.setup = function (readOnlyInstancOf, readWriteInstanceOf) {
        // setup 
        //console.log('setting up');
        return new Promise((resolve, reject) => {
            task(function *(){
                if(readOnlyInstancOf) {
                    let res = yield addPool('READ_ONLY', readOnlyInstancOf);
                }
        
                if(readWriteInstanceOf) {
                    let res = yield addPool('READ_WRITE', readWriteInstanceOf);
                }
                resolve('done');
            });
        });
    };

    _helper.prototype.tearDown = function (){
        //console.log('tearing down');
        return closeAllConnections();
    };

    _helper.prototype.runQuery = function (sql, inputs){
        // this function will ultimately be smarter but for now just send through the required logic
        //console.log('making query');

        return new Promise((resolve, reject) => {
            if(_.startsWith(_.toLower(sql), 'select') || _.startsWith(_.toLower(sql), 'set')) {
                runSelectQuery(sql, inputs, function(err, rows){
                    if(err) {
                        reject(err);
                    } else {
                        resolve(rows);
                    }
                });
            } else if(_.startsWith(_.toLower(sql), 'insert')) {
                runInsertQuery(sql, inputs, function(err, res){
                    if(err) {
                        reject(err);
                    } else {
                        resolve(res);
                    }
                });
            } else {
                runUpdateDeleteQuery(sql, inputs, function(err, res){
                    if(err) {
                        reject(err);
                    } else {
                        resolve(res);
                    }
                });
            }
        });
    };

    function closeAllConnections() {
        return new Promise((resolve, reject) => {
            poolCluster.end(function (err) {
                if(err) console.log(err);

                resolve('cluster closed');
            });
        });
    }

    function task(generator) {
        let iterator = generator(); // create generator object
        recursivelyNext();
    
        // this functions keeps calling next() if a promise is yielded
        function recursivelyNext(data) {
            let yielded = iterator.next.apply(iterator, arguments); // { value: Any, done: Boolean }
    
            if (isPromise(yielded.value)) {
                yielded.value.then((data) => {
                    recursivelyNext(data);
                });
            }
        }
    }
      
    function isPromise(val) {
        return val && typeof val.then === 'function';
    }

    function addPool(poolId, instanceOf) {
        return new Promise((resolve, reject) => {
            configurationManager.getCredentials('DATABASE', instanceOf, function(err, credentials){
                if(err){ 
                    reject(err);
                } else {
                    let poolConfig = {
                        connectionLimit : 10,
                        host            : process.env.LOCAL_DEVELOPMENT == 'yes' ? 'localhost' : credentials.host,
                        port            : process.env.LOCAL_DEVELOPMENT == 'yes' ? getPortForDevEnvironment(credentials.host) : (credentials.port ? credentials.port : 3306),
                        user            : credentials.secret_username ? credentials.secret_username : credentials.user,
                        password        : credentials.secret_password ? credentials.secret_password : credentials.pass,
                        database        : credentials.dbname,
                        dateStrings     : true,
                        acquireTimeout  : 20000,
                        waitForConnections : true,
                        multipleStatements: true
                    };
                    //console.log(poolConfig);
                    poolCluster.add(poolId, poolConfig);
                    resolve('ADDED');
                }
                
            });
        });
    }

    function getPortForDevEnvironment(host) {
        let port = 3306;
        if (host.indexOf('rds-production') >= 0) {
            port = 13306;
        } else if (host.indexOf('rds-replica') >= 0) {
            port = 13307;
        } else if (host.indexOf('rds-staging') >= 0) {
            port = 13308;
        }
        return port;
    }

    function runSelectQuery(sql, inputs, cb) {
        poolCluster.getConnection('READ_ONLY', function(err, connection){
            if(err) {
                cb(err, null);
            } else {
                // get SQL
                let fullSQL = makeSqlString(sql, inputs);
                connection.query(fullSQL, function(err, rows, fields){
                    connection.release();
                    if(err) {
                        cb(err, null);
                    } else {
                        cb(null, rows);
                    }
                });
            }
        });
    }

    function runUpdateDeleteQuery(sql, inputs, cb) {
        poolCluster.getConnection('READ_WRITE', function(err, connection){
            if(err) {
                cb(err, null);
            } else {
                // get SQL
                let fullSQL = makeSqlString(sql, inputs);
                connection.query(fullSQL, function(err, res){
                    connection.release();
                    if(err) {
                        cb(err, null);
                    } else {
                        console.log(`UPDATED / DELETED ${res.affectedRows} rows`);
                        cb(null, res);
                    }
                });
            }
        });
    }

    function runInsertQuery(sql, inputs, cb) {
        poolCluster.getConnection('READ_WRITE', function(err, connection){
            if(err) {
                cb(err, null);
            } else {
                // split inputs into chunks of 500 rows / could make this smarter too I guess
                let sizeOfChunks = 500;
                let chunks = _.chunk(inputs, sizeOfChunks);

                async.concatSeries(
                    chunks,
                    function(chunk, complete) {
                        // get SQL
                        let fullSQL = makeSqlString(sql, chunk);
                        connection.query(fullSQL, function(err, res){
                            if(err) {
                                //console.log(err);
                                complete(err);
                            } else {
                                console.log(`INSERTED ${res.affectedRows} rows`);
                                complete(null, res);
                            }
                        });
                    },
                    function(error, results) {
                        connection.release();
                        cb(error, results);
                    }
                );
            }
        });
    }

    function makeSqlString(sql, inputs) {
        let fullSQL = SqlString.format(sql, inputs, false, 'local');
        //console.log(fullSQL);
        return fullSQL;
    }
    
    return _helper;
    
})();
    
module.exports = _helper;
