'use strict';

let AWS = require('aws-sdk');
let SQSQueue = require('./sqs_queue.js');

class SQSHelper {
    /**
     * @constructor
     * @param {object=} params additional configuration options
     * @param {string=} params.profile aws profile name. If this is not specified, the profile name specified in AWS_PROFILE envrinment variable will be used. If AWS_PROFILE environment is not available, credentials will be retrieved from environment variables.
     * @param {string=} params.region The default value is 'ap-southeast-2'. If this is specified, it will use it instead.
     */
    constructor(params) {
        let creds = null;
        if (typeof(params) == 'object' && params.hasOwnProperty('profile')) {
            creds = new AWS.SharedIniFileCredentials({profile: params.profile});
        } else if (process.env.hasOwnProperty('AWS_PROFILE')) {
            creds = new AWS.SharedIniFileCredentials({profile: process.env.aws_profile});
        } else {
            creds = new AWS.EnvironmentCredentials('AWS');
        }

        this._awsConfig = {
            credentials: creds,
            region: 'ap-southeast-2'
        };

        this._sqs = null;
        this._queues = {}
    }

    /**
     * create AWS.SQS object
     * 
     * @public
     * @returns {object} AWS.SQS object
     */
    getSQS() {
        if (!this._sqs) {
            this._sqs = new AWS.SQS(this._awsConfig);
        }

        return this._sqs;
    }

    /**
     * create a S3Bucket object and return it
     * 
     * @public
     * @param {string} queueName name of the queue that you want to work with
     * @returns {object} SQSQueue object
     */
    getQueue(queueName) {
        let queue = null;

        if (this._queues.hasOwnProperty(queueName)) {
            queue = this._queues[queueName];
        } else {
            queue = new SQSQueue(queueName, this);
        }

        return queue;
    }
}

module.exports = SQSHelper;
