'use strict';

const request = require('request');
const async = require('async');
const _ = require('lodash');

class SugarcrmHelper {
    constructor(url, user, pwd) {
        this.url = url;
        this.user = user;
        this.pwd = pwd;
        this.token = null;
    }

    /**
     * Used to login to the CRM
     * 
     * @param {function} callback 
     */
    login(callback) {
        var data = {
            grant_type: "password",
            client_id: "sugar",
            client_secret: "",
            username: this.user,
            password: this.pwd,
            platform: "api"
        };
    
        request({
            uri: this.url + '/oauth2/token',
            method: 'POST',
            json: true,
            formData: data,
            resolveWithFullResponse: true
        }, (err, req, body) => {
            if (err) {
                return callback(err, body);
            } else if (typeof (body) == 'object' && body.hasOwnProperty('error') && body.hasOwnProperty('error_message')) {
                return callback(`${body['error_message']}`, body);
            } else if (typeof (body) == 'object' && body.hasOwnProperty('access_token')) {
                this.token = body['access_token'];
                return callback(null, body);
            } else {
                return callback('unknown_error', body);
            }
        });
    }

    /**
     * Used to logout of the current session
     * 
     * @param {function} callback 
     */
    logout(callback) {
        request({
            url: this.url + '/oauth2/logout',
            method: 'POST',
            headers: {
                'oauth-token': this.token
            },
            json: true,
        }, function (err, req, body) {
            if (err) {
                return callback(err, body);
            } else if (body.success == true) {
                return callback(null, body);
            } else {
                return callback('unknown_error', body);
            }
        });
    }

    /**
     * Used to get all the data from one module and ID
     * 
     * @param {string} module 
     * @param {string} id 
     * @param {function} callback 
     */
    getOne(module, id, callback) {
        request({
            url: this.url + '/' + module + '/' + id,
            method: 'GET',
            headers: {
                'oauth-token': this.token
            },
            json: true,
        }, function (err, req, body) {
            if (err) {
                return callback(err, body);
            } else if (typeof (body) == 'object' && body.hasOwnProperty('error') && body.hasOwnProperty('error_message')) {
                return callback(`${body['error_message']}`, body);
            } else if (typeof (body) == 'object' && body.hasOwnProperty('id')) {
                return callback(null, body);
            } else {
                return callback('unknown_error', body);
            }
        });
    }

    /**
     * Used to get the related records of a module
     * 
     * @param {string} module 
     * @param {string} id 
     * @param {string} linkName
     * @param {function} callback 
     */
    getRelated(module, id, linkName, callback) {
        request({
            url: this.url + '/' + module + '/' + id + '/link/' + linkName,
            method: 'GET',
            headers: {
                'oauth-token': this.token
            },
            json: true,
        }, function (err, req, body) {
            if (err) {
                return callback(err, body);
            } else if (typeof (body) == 'object' && body.hasOwnProperty('error') && body.hasOwnProperty('error_message')) {
                return callback(`${body['error_message']}`, body);
            } else if (typeof (body) == 'object' && body.hasOwnProperty('records')) {
                return callback(null, body);
            } else {
                return callback('unknown_error', body);
            }
        });
    }

    /**
     * Used to get one record and then its related records based on the specified relatedModules
     * 
     * examples of relatedModules when to search MOJO_Services module
     * [
     *      {
     *          objectName: 'Accounts', 
     *          objectReferenceKeyName: 'MOJO_Services.accounts_mojo_services_1accounts_ida'
     *      },
     *      {
     *          linkName: 'contacts',
     *          objectName: 'Accounts', 
     *          objectReferenceKeyName: 'MOJO_Services.accounts_mojo_services_1accounts_ida',
     *          selectFirstRecordOnly: true
     *      },
     *      {
     *          objectName: 'MOJO_Sites',
     *          objectReferenceKeyName: 'MOJO_Services.mojo_services_mojo_sitesmojo_sites_ida'
     *      },
     *      {
     *          objectName: 'MOJO_Payment_Detail',
     *          objectReferenceKeyName: 'MOJO_Services.mojo_services_mojo_payment_detailmojo_payment_detail_ida'
     *      }
     * ]
     * 
     * @param {string} module 
     * @param {string} id 
     * @param {object} relatedModules 
     * @param {function} callback 
     */
    getOneAndRelated(module, id, relatedModules, callback) {
        let record = null;
        this.getOne(module, id, (err, body) => {
            if (err) {
                console.error(body);
                callback(err, null);
            } else {
                record = {};
                record[body._module] = body;

                async.each(
                    relatedModules, 
                    (related, done) => {
                        let relatedKey = _.get(record, related['objectReferenceKeyName'], null);

                        if (relatedKey) {
                            if (related.hasOwnProperty('linkName')) {
                                this.getRelated(related['objectName'], relatedKey, related['linkName'], (err, body) => {
                                    if (err) {
                                        console.error(body);
                                        done(err);
                                    } else {
                                        if (_.isArray(body['records']) && body['records'].length > 0) {
                                            let linkObjectName = _.get(body, 'records[0]._module');
                                            if (related.hasOwnProperty('selectFirstRecordOnly') && related['selectFirstRecordOnly']) {
                                                record[linkObjectName] = body['records'][0];
                                            } else {
                                                record[linkObjectName] = body['records'];
                                            }
                                            done(null);
                                        } else {
                                            done(`no ${related['linkName']} records of ${related['objectName']} ${relatedKey}`);
                                        }
                                        
                                    }
                                });
                            } else {
                                this.getOne(related['objectName'], relatedKey, (err, body) => {
                                    if (err) {
                                        console.log('kfslkfjsldkjfsdlkfjsdlkfj');
                                        console.error(err);
                                        done(err);
                                    } else {
                                        record[body._module] = body;
                                        done(null);
                                    }
                                });
                            }
                        } else {
                            if (related.hasOwnProperty('errorOnEmpty') && related['errorOnEmpty']) {
                                done(`no key identified by ${related['objectReferenceKeyName']}`);
                            } else {
                                done(null);
                            }
                        }
                    },
                    (err) => {
                        if (err) {
                            callback(err, null);
                        } else {
                            callback(null, record);
                        }
                    }
                );
            }
        });
    }

    /**
     * Used to get the data from one module with a filter
     * 
     * @param {string} module 
     * @param {object} data 
     * @param {function} callback 
     */
    getFilterData(module, data, callback) {
        request({
            url: this.url + '/' + module + '/filter',
            method: 'POST',
            headers: {
                'oauth-token': this.token
            },
            json: true,
            body: data,
        }, function (err, req, body) {
            if (err) {
                return callback(err, body);
            } else if (typeof (body) == 'object' && body.hasOwnProperty('error') && body.hasOwnProperty('error_message')) {
                return callback(`${body['error_message']}`, body);
            } else if (typeof (body) == 'object' && body.hasOwnProperty('records')) {
                return callback(null, body);
            } else {
                return callback('unknown_error', body);
            }
        });
    }

    /**
     * Used to create a new record in the CRM
     * 
     * @param {string} module 
     * @param {object} data 
     * @param {function} callback 
     */
    createRecord(module, data, callback) {
        request({
            url: this.url + '/' + module,
            method: 'POST',
            headers: {
                'oauth-token': this.token,
            },
            json: true,
            body: data,
        }, function (err, req, body) {
            if (err) {
                return callback(err, body);
            } else if (typeof (body) == 'object' && body.hasOwnProperty('error') && body.hasOwnProperty('error_message')) {
                return callback(`${body['error_message']}`, body);
            } else if (typeof (body) == 'object' && body.hasOwnProperty('id')) {
                return callback(null, body);
            } else {
                return callback('unknown_error', body);
            }
        });
    }

    /**
     * update a record in the CRM
     * 
     * @param {string} module 
     * @param {string} id 
     * @param {object} data 
     * @param {function} callback 
     */
    updateRecord(module, id, data, callback) {
        request({
            url: this.url + '/' + module + '/' + id,
            method: 'PUT',
            headers: {
                'oauth-token': this.token,
            },
            json: true,
            body: data,
        }, function (err, req, body) {
            if (err) {
                return callback(err, body);
            } else if (typeof (body) == 'object' && body.hasOwnProperty('error') && body.hasOwnProperty('error_message')) {
                return callback(`${body['error_message']}`, body);
            } else if (typeof (body) == 'object' && body.hasOwnProperty('id')) {
                return callback(null, body);
            } else {
                return callback('unknown_error', body);
            }
        });
    }

    /**
     * delete a record from the CRM
     * 
     * @param {string} module 
     * @param {string} id 
     * @param {function} callback 
     */
    deleteRecord(module, id, callback) {
        request({
            url: this.url + '/' + module + '/' + id,
            method: 'DELETE',
            headers: {
                'oauth-token': this.token,
            },
            json: true,
        }, function (err, req, body) {
            if (err) {
                return callback(err, body);
            } else if (body.success == true) {
                return callback(null, body);
            } else {
                return callback('unknown_error', body);
            }
        });
    }

    /**
     * subscribe to a record in the CRM
     * 
     * @param {string} module 
     * @param {string} id 
     * @param {function} callback 
     */
    followRecord(module, id, callback) {
        this.recordAction(module, id, 'subscribe', function (body, err) {
            callback(body, err);
        });
    }

    /**
     * favourite to a record in the CRM
     * 
     * @param {string} module 
     * @param {string} id 
     * @param {function} callback 
     */
    favoriteRecord(module, id, callback) {
        recordAction(module, id, 'favorite', function (body, err) {
            callback(body, err);
        });
    }

    /**
     * unsubsribe to a record in the CRM
     * 
     * @param {string} module 
     * @param {string} id 
     * @param {function} callback 
     */
    unfollowRecord(module, id, callback) {
        recordAction(module, id, 'unsubscribe', function (body, err) {
            callback(body, err);
        });
    }

    /**
     * unfavourite to a record in the CRM
     * 
     * @param {*} module 
     * @param {*} id 
     * @param {*} callback 
     */
    unfavoriteRecord(module, id, callback) {
        recordAction(module, id, 'unfavorite', function (body, err) {
            callback(body, err);
        });
    }

    /**
     * Search the CRM globally
     * 
     * @param {object} searchData 
     * @param {function} callback 
     */
    search(searchData, callback) {
        searchData = this.buildQueryString(searchData);
        if (!searchData) {
            callback(null, { "action": "fail", "msg": "Failed to convert search data to url" });
        }
        request({
            url: this.url + '/search/?' + searchData,
            method: 'GET',
            headers: {
                'oauth-token': this.token,
            },
            json: true,
        }, function (err, req, body) {
            if (body.success == true) {
                return callback(body)
            } else {
                return callback(body, err);
            }
        });
    }

    /**
     * Check if there is duplicated record based on the search fields given by 'data' parameter
     * 
     * @param {string} module 
     * @param {object} data 
     * @param {function} callback 
     */
    duplicateCheck(module, data, callback) {
        request({
            url: this.url + '/' + module + '/duplicateCheck',
            method: 'POST',
            headers: {
                'oauth-token': this.token,
            },
            json: true,
            body: data,
        }, function (err, req, body) {
            if (err) {
                return callback(err, body);
            } else if (typeof (body) == 'object' && body.hasOwnProperty('error') && body.hasOwnProperty('error_message')) {
                return callback(`${body['error_message']}`, body);
            } else if (typeof (body) == 'object' && body.hasOwnProperty('records')) {
                return callback(null, body);
            } else {
                return callback('unknown_error', body);
            }

        });
    }

    /**
     * subscribe/unsubscribe/favourite/unfavourite
     * 
     * @param {string} module 
     * @param {string} id 
     * @param {string} type 
     * @param {function} callback 
     */
    recordAction(module, id, type, callback) {
        request({
            url: this.url + '/' + module + '/' + id + '/' + type,
            method: 'POST',
            headers: {
                'oauth-token': this.token,
            },
            json: true,
        }, function (err, req, body) {
            if (body.success == true) {
                return callback(body)
            } else {
                return callback(body, err);
            }
        });
    }

    /**
     * Build the query strings.
     * 
     * @param {object} data 
     */
    buildQueryString(data) {
        var components = [];
        for (var i in data) {
            components.push('&');
            components.push(encodeURIComponent(i) + '=' + encodeURIComponent(data[i]));
        }
        return components;
    }
}

module.exports = SugarcrmHelper;